/*
 * Copyright 2009 Johnson Consulting Services
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package com.johnsoncs.gwt.application.client.model;

import java.io.Serializable;

import com.johnsoncs.gwt.application.client.model.Data.DataValue;
import com.johnsoncs.gwt.application.client.model.Data.Key;
import com.johnsoncs.gwt.application.client.model.Data.Property;
import com.johnsoncs.gwt.application.client.model.Data.Value;

/**
 * @author wilj
 *
 */
@SuppressWarnings("unchecked")
public class Model implements Serializable {
	public interface QueryCallback<T> {
		void onResult(QueryPath path, T result);
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private ModelDefinition definition;

	private Data root;

	protected Model() {
		// do nothing
	}

	public Model(final ModelDefinition definition, final Data root) {
		this.definition = definition;
		this.root = root;
	}

	public <T> T get(final QueryPath path) {
		return (T) root.query(path);
	}

	/**
	 * @return the root
	 */
	public Data getRoot() {
		return root;
	}

	public <T> void query(final QueryPath path, final QueryCallback<T> callback) {
		queryRelative(root, path, callback);
	}

	public <T> void query(final String path, final QueryCallback<T> callback) {
		query(QueryPath.parse(path), callback);
	}

	@SuppressWarnings("unchecked")
	private <T> void queryRelative(final Data branch, final QueryPath path, final QueryCallback<T> callback) {
		Data d = branch;

		for (int i = 0; i < path.size(); i++) {
			final Key key = path.get(i);
			if (key.equals(Data.WILDCARD_KEY)) {
				final QueryPath relative = path.subPath(i + 1, path.size());
				for (final Property p : d) {
					// TODO this won't work with wildcarded leafnodes either
					if (p.getValueType().equals(Data.class)) {
						queryRelative((Data) p.getValue(), relative, callback);
					}
				}
			} else if (i < path.size() - 1) {
				d = d.get(key);
			} else {
				final QueryPath resultPath = d.getQueryPath();
				// TODO this won't work for leaf nodes that are wildcarded, e.g. myobj.mysubobj.*, probably need to enable leaf nodes to track their keys
				resultPath.add(key);
				callback.onResult(resultPath, (T) d.get(key));
			}
		}
	}

	public void set(final QueryPath path, final Data value) {
		set(path, new Data.DataValue(value));
	}

	public void set(final QueryPath path, final Integer value) {
		set(path, new Data.IntegerValue(value));
	}

	public void set(final QueryPath path, final String value) {
		set(path, new Data.StringValue(value));
	}

	public void set(final QueryPath path, final Value value) {
		definition.ensurePath(root, path, value instanceof DataValue);
		final QueryPath q = path.subPath(0, path.size() - 1);
		final Data d = root.query(q);
		d.set(path.get(path.size() - 1), value);
	}

	/**
	 * @param root the root to set
	 */
	public void setRoot(final Data root) {
		this.root = root;
	}
}
