/*
 * Copyright 2009 Johnson Consulting Services
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * 
 */
package com.johnsoncs.gwt.application.client.validation;

import com.johnsoncs.gwt.application.client.validation.constraints.LengthConstraint;
import com.johnsoncs.gwt.application.client.validation.constraints.RangeConstraint;
import com.johnsoncs.gwt.application.client.validation.constraints.RequiredConstraint;

/**
 * @author wilj
 *
 */
public abstract class ValidatorBuilder {
	protected class Field {
		private final String path;

		public Field(final String path) {
			this.path = path;
		}

		public Field addConstraint(final Constraint constraint) {
			validator.addConstraint(path, constraint);
			return this;
		}

		public Field between(final Comparable<?> minValue, final Comparable<?> maxValue) {
			return addConstraint(new RangeConstraint(minValue, maxValue));
		}

		public Field length(final Integer minLength, final Integer maxLength) {
			return addConstraint(new LengthConstraint(minLength, maxLength));
		}

		public Field required() {
			return addConstraint(new RequiredConstraint());
		}

	}

	private final ValidationContext context;

	private Validator validator;

	public ValidatorBuilder(final ValidationContext context) {
		this.context = context;
	}

	public Validator build() {
		if (validator == null) {
			validator = new Validator(context);
			defineConstraints();
		}
		return validator;
	}

	protected Field define(final String queryPath) {
		return new Field(queryPath);
	}

	protected abstract void defineConstraints();
}
