/*
 * Copyright 2009 Johnson Consulting Services
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.johnsoncs.gwt.widgets.client.dynamictable.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

import com.johnsoncs.gwt.widgets.client.dynamictable.ColumnDefinition;
import com.johnsoncs.gwt.widgets.client.dynamictable.ColumnPicker;
import com.johnsoncs.gwt.widgets.client.dynamictable.DynamicTable;
import com.johnsoncs.gwt.widgets.client.dynamictable.ColumnDefinition.ColumnAttribute;
import com.johnsoncs.gwt.widgets.client.themes.Theme;
import com.johnsoncs.gwt.widgets.client.util.Callback;

/**
 * Defines an action that displays a table settings popup.  Currently the only settings that can be changed are which columns are visible.
 * @author wilj
 *
 * @param <T> the generic type with which the table structure is associated
 */
public class ShowSettingsAction<T> implements ActionHandler<T> {
	public static final ColumnAttribute PINNED = new ColumnAttribute() {
	};

	/**
	 * @see com.johnsoncs.gwt.widgets.client.dynamictable.actions.ActionHandler#isEnabledFor(com.johnsoncs.gwt.widgets.client.dynamictable.DynamicTable)
	 */
	@Override
	public boolean isEnabledFor(final DynamicTable<T> table) {
		return true;
	}

	/**
	 * @see com.johnsoncs.gwt.widgets.client.dynamictable.actions.ActionHandler#onAction(com.johnsoncs.gwt.widgets.client.dynamictable.DynamicTable)
	 */
	@Override
	public void onAction(final DynamicTable<T> table) {
		final List<ColumnDefinition<T>> cols = new ArrayList<ColumnDefinition<T>>();
		for (final ColumnDefinition<T> col : table.getDefinition().getColumns()) {
			if (!col.hasAttribute(PINNED.getClass())) {
				cols.add(col);
			}
		}
		final ColumnPicker<T> picker = new ColumnPicker<T>(cols, Theme.INSTANCE
				.getTableMessages().getSelectColumns(),
				new ColumnPicker.IsSelectedAdapter<T>() {
			@Override
			public boolean isSelected(final ColumnDefinition<T> column) {
				return column.isVisible();
			}
		});

		picker.show(new Callback<Boolean>() {
			@Override
			public void exec(final Boolean value) {
				if (value && picker.isSelectionChanged()) {
					for (final Entry<ColumnDefinition<T>, Boolean> e : picker.getSelection().entrySet()) {
						e.getKey().setVisible(e.getValue());
					}
					table.redraw();
				}
			}
		});
	}

}
